package com.stormtales.notevault.data.entities;

import androidx.room.Entity;
import androidx.room.Ignore;
import androidx.room.PrimaryKey;
import com.stormtales.notevault.data.sync.SyncStatus;
import com.stormtales.notevault.network.sync.models.SongModel;

import java.time.LocalDateTime;
import java.util.Objects;

@Entity
public class Song {
    @PrimaryKey(autoGenerate = true)
    private int localID;

    private String serverID;
    private SyncStatus syncStatus;
    private LocalDateTime syncTime;

    /**Meta Data of Song*/
    private String title;
    private String composer;
    private String genre;
    private int year;

    @Ignore
    public Song(String title, String composer, String genre, int year) {
        this.title = title;
        this.composer = composer;
        this.genre = genre;
        this.year = year;
        this.syncStatus = SyncStatus.CREATED;
    }

    @Ignore
    public Song(SongModel songModel) {
        this.serverID = songModel.getServerID();
        this.title = songModel.getTitle();
        this.composer = songModel.getComposer();
        this.genre = songModel.getGenre();
        this.year = songModel.getYear();
        this.syncStatus = SyncStatus.SYNCED;
        this.syncTime = LocalDateTime.now();
    }

    public Song() {
    }

    public int getLocalID() {
        return localID;
    }

    public void setLocalID(int localID) {
        this.localID = localID;
    }

    public String getServerID() {
        return serverID;
    }

    public void setServerID(String serverID) {
        this.serverID = serverID;
    }

    public SyncStatus getSyncStatus() {
        return syncStatus;
    }

    public void setSyncStatus(SyncStatus syncStatus) {
        this.syncStatus = syncStatus;
    }

    public LocalDateTime getSyncTime() {
        return syncTime;
    }

    public void setSyncTime(LocalDateTime syncTime) {
        this.syncTime = syncTime;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getComposer() {
        return composer;
    }

    public void setComposer(String composer) {
        this.composer = composer;
    }

    public String getGenre() {
        return genre;
    }

    public void setGenre(String genre) {
        this.genre = genre;
    }

    public int getYear() {
        return year;
    }

    public void setYear(int year) {
        this.year = year;
    }

    @Override
    public boolean equals(Object o) {
        if (o == null || getClass() != o.getClass()) return false;
        Song song = (Song) o;
        return localID == song.localID && Objects.equals(serverID, song.serverID);
    }

    @Override
    public int hashCode() {
        return Objects.hash(localID, serverID);
    }
}
