package com.stormtales.notevault.ui.login;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.widget.*;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import com.stormtales.notevault.R;
import com.stormtales.notevault.network.APICallback;
import com.stormtales.notevault.network.auth.AuthService;
import org.jetbrains.annotations.NotNull;

public class LoginDialog extends DialogFragment {

    private LoginViewModel loginViewModel;
    private boolean isLoginMode = true;
    private Dialog dialog;

    private Button loginButton;
    private ProgressBar loadingProgressBar;
    private EditText editTextUsername;
    private EditText editTextPassword;
    private EditText editTextEmail;
    private TextView textViewTitle;
    private TextView textViewSwitch;

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);

    }

    public LoginDialog(LoginViewModel loginViewModel) {
        this.loginViewModel = loginViewModel;
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        AlertDialog.Builder builder = new AlertDialog.Builder(requireContext());
        LayoutInflater inflater = requireActivity().getLayoutInflater();
        View view = inflater.inflate(R.layout.fragment_login_dialog, null);

        textViewTitle = view.findViewById(R.id.textViewTitle);
        editTextUsername = view.findViewById(R.id.editTextUsername);
        editTextEmail = view.findViewById(R.id.editTextEmail);
        editTextPassword = view.findViewById(R.id.editTextPassword);
        loginButton = view.findViewById(R.id.buttonAction);
        textViewSwitch = view.findViewById(R.id.textViewSwitch);
        loadingProgressBar = view.findViewById(R.id.loading);
        // Handle action button click


        // Toggle between Login and Registration
        textViewSwitch.setOnClickListener(v -> {
            isLoginMode = !isLoginMode;
            textViewTitle.setText(isLoginMode ? "Login" : "Register");
            editTextUsername.setVisibility(isLoginMode ? View.GONE : View.VISIBLE);
            loginButton.setText(isLoginMode ? "Login" : "Register");
            textViewSwitch.setText(isLoginMode ? "Don't have an account? Register" : "Already have an account? Login");
        });

        builder.setView(view);
        dialog = builder.create();



        TextWatcher afterTextChangedListener = new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                loginViewModel.updateLoginData(editTextUsername.getText().toString(), editTextPassword.getText().toString());
            }
        };

        editTextUsername.addTextChangedListener(afterTextChangedListener);
        editTextPassword.addTextChangedListener(afterTextChangedListener);
        editTextPassword.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
                if(actionId == EditorInfo.IME_ACTION_DONE) {
                    loginViewModel.performLogin(editTextUsername.getText().toString(), editTextPassword.getText().toString(), LoginDialog.this::onSuccessFullLogin);
                }
                return false;
            }
        });

        loginButton.setOnClickListener(v -> {
            loadingProgressBar.setVisibility(View.VISIBLE);
            String email = editTextEmail.getText().toString();
            String password = editTextPassword.getText().toString();
            if (isLoginMode) {
                // Handle login logic
                this.loginViewModel.performLogin(email, password, this::onSuccessFullLogin);
            } else {
                // Handle registration logic
                String username = editTextUsername.getText().toString();
                this.loginViewModel.performRegistration(email, password, username, this::onSuccessFullRegistration);
            }
        });
        return dialog;
    }

    private void onSuccessFullRegistration() {
        Toast.makeText(getContext(), "Successfully registered", Toast.LENGTH_LONG).show();
        this.isLoginMode = true;
        textViewTitle.setText(isLoginMode ? "Login" : "Register");
        editTextUsername.setVisibility(isLoginMode ? View.GONE : View.VISIBLE);
        loginButton.setText(isLoginMode ? "Login" : "Register");
        textViewSwitch.setText(isLoginMode ? "Don't have an account? Register" : "Already have an account? Login");
    }

    @Override
    public void onViewCreated(@NonNull @NotNull View view, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        loginViewModel.getLoginFormState().observe(getViewLifecycleOwner(), new Observer<LoginFormState>() {
            @Override
            public void onChanged(LoginFormState loginFormState) {
                if(loginFormState == null) return;
                loginButton.setEnabled(loginFormState.isDataValid());
                if(loginFormState.getUsernameError() != null) {
                    editTextUsername.setError(getString(loginFormState.getUsernameError()));
                }
                if(loginFormState.getPasswordError() != null) {
                    editTextPassword.setError(getString(loginFormState.getPasswordError()));
                }
            }
        });
    }

    void onSuccessFullLogin() {
        this.dialog.dismiss();
    }

}
